﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace WindowsFormsApp37
{
    public partial class Form1 : Form
    {
        private DataSet singleTableDataSet;
        private DataSet twoTableDataSet;
        private const int parentRowCount = 10;
        private const int childRowCount = 1000;

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            singleTableDataSet = this.GetRecursiveDataSetWithOneTable();
            this.ultraTree1.InitializeDataNode += UltraTree1_InitializeDataNode;
            this.ultraTree1.SetDataBinding(singleTableDataSet, "Root Table");

            twoTableDataSet = this.GetRecursiveDataSetWithTwoTables();            
            this.ultraTree2.SetDataBinding(twoTableDataSet, "Root Table");
        }

        private void UltraTree1_InitializeDataNode(object sender, Infragistics.Win.UltraWinTree.InitializeDataNodeEventArgs e)
        {
            if (e.Node.Level == 0)
            {
                var parentId = e.Node.Cells["ParentId"].Value;
                if (!(parentId is DBNull))
                    e.Node.Visible = false;
            }
        }

        #region Single Table
        private DataSet GetRecursiveDataSetWithOneTable()
        {
            DataSet ds = new DataSet();
            var table = CreateSingleTableWithData();
            table.TableName = "Root table";
            ds.Tables.Add(table);
            ds.Relations.Add(table.Columns["Id"], table.Columns["ParentID"]);
            return ds;
        }

        private DataTable CreateSingleTableWithData()
        {
            // Create a table with parentRowCount root-level rows and everything else is child rows.
            DataTable dt = new DataTable();
            var idColumn = dt.Columns.Add("Id", typeof(int));
            idColumn.Unique = true;
            dt.Columns.Add("ParentID", typeof(int));
            dt.Columns.Add("Name", typeof(string));

            // In this case, both the parent and child data are in the same table. 
            this.AddParentData(dt);
            this.AddChildData(dt);
            return dt;
        } 
        #endregion // Single Table

        #region Two Tables
        private DataSet GetRecursiveDataSetWithTwoTables()
        {
            DataSet ds = new DataSet();
            var parentTable = CreateRootTableWithData();
            parentTable.TableName = "Root table";
            ds.Tables.Add(parentTable);

            var childTable = CreateChildTableWithData();
            childTable.TableName = "Child table";
            ds.Tables.Add(childTable);

            // We have to specify false here for CreateConstraints, because some of the children have 
            // parent that are in a different table. 
            ds.Relations.Add("Parent To Child", parentTable.Columns["Id"], childTable.Columns["ParentID"], false);            
            ds.Relations.Add("Child To Child", childTable.Columns["Id"], childTable.Columns["ParentID"], false);
            return ds;
        }

        private DataTable CreateRootTableWithData()
        {
            // Create a table with parentRowCount root-level rows 
            DataTable dt = new DataTable();
            var idColumn = dt.Columns.Add("Id", typeof(int));
            idColumn.Unique = true;

            // We don't really even need this column for the root level. You could exclude it from your Select query.
            // But I am leaving it in here, since it makes the ading of rows easier in code. 
            dt.Columns.Add("ParentID", typeof(int));

            dt.Columns.Add("Name", typeof(string));

            // In this case, just add the parent data
            this.AddParentData(dt);
            return dt;
        }

        private DataTable CreateChildTableWithData()
        {
            // Create a table with parentRowCount root-level rows 
            DataTable dt = new DataTable();
            var idColumn = dt.Columns.Add("Id", typeof(int));
            idColumn.Unique = true;
            dt.Columns.Add("ParentID", typeof(int));
            dt.Columns.Add("Name", typeof(string));

            // In this case, just add the child data
            this.AddChildData(dt);
            return dt;
        }
        #endregion // Two Tables


        #region Create the actual data
        private void AddParentData(DataTable dt)
        {
            for (int i = 0; i < parentRowCount; i++)
            {
                string name = DummyDataCreator.RandomString();
                // The ParentId is null here, to indicate that this is a root-level row
                dt.Rows.Add(new object[] { i, null, name });
            }
        }

        private void AddChildData(DataTable dt)
        {
            for (int i = parentRowCount; i < childRowCount; i++)
            {
                string name = DummyDataCreator.RandomString();

                // Pick a random parent row.
                // By using i-1 here, we ensure that the parent will be a row that was already created. 
                int parentId = DummyDataCreator.RandomInt(0, i - 1);
                dt.Rows.Add(new object[] { i, parentId, name });
            }
        }
        #endregion // Create the actual data
    }
}
